-- ========================================================================== --
--                    Eska Encounter Nameplates                               --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/boss-mod-nameplates        --
-- ========================================================================== --
Scorpio                   "BossModNameplates"                            "1.0.0"
-- ========================================================================== --
namespace "BMN"
-- ===================== Boss Mod Handlers =====================================
_BossModHandlers = {}

-- ==========================[[ Logger ]]=======================================
Log               = Logger("BossModNameplates")

Trace             = Log:SetPrefix(1, "|cffa9a9a9[BossModNameplates:Trace]|r", true)
Debug             = Log:SetPrefix(2, "|cff808080[BossModNameplates:Debug]|r", true)
Info              = Log:SetPrefix(3, "|cffffffff[BossModNameplates:Info]|r", true)
Warn              = Log:SetPrefix(4, "|cffffff00[BossModNameplates:Warn]|r", true)
Error             = Log:SetPrefix(5, "|cffff0000[BossModNameplates:Error]|r", true)
Fatal             = Log:SetPrefix(6, "|cff8b0000[BossModNameplates:Fatal]|r", true)

Log.LogLevel = 3
Log:AddHandler(print)
-- =============================================================================


function OnLoad(self)
  _DB = SVManager("BossModNameplatesDB")

  _DB:SetDefault("General", {
    bigWigsSupport = true,
    dbmSupport = true,
  })

  _DB:SetDefault("Nameplates", {
    height = 64,
    width = 64,
    showBorder = true,
    offsetX = 0,
    offsetY = -10,
    borderColor = { r = 0, g = 0, b = 0},
    anchorFrom = "BOTTOM",
    anchorTo = "TOP",
  })

  -- Apply the DB to Nameplate
  Nameplate.defaultWidth = _DB.Nameplates.width
  Nameplate.defaultHeight = _DB.Nameplates.height
  Nameplate.defaultOffsetX = _DB.Nameplates.offsetX
  Nameplate.defaultOffsetY = _DB.Nameplates.offsetY
  Nameplate.defaultAnchorFrom = _DB.Nameplates.anchorFrom
  Nameplate.defaultAnchorTo = _DB.Nameplates.anchorTo

  self.unitsCache = {}
  self.auras = Dictionary()

  self:InitiliazeBossMods()
end

function OnEnable(self)
  -- BigWigs events
  self:RegisterEvent("BigWigs_ShowNameplateAura", "BigWigsShowAura")
  self:RegisterEvent("BigWigs_HideNameplateAura", "BigWigsHideAura")
  self:RegisterEvent("BigWigs_EnableFriendlyNameplates", "BigWigsEnableFriendlyNameplates")
  self:RegisterEvent("BigWigs_DisableFriendlyNameplates", "BigWigsDisableFriendlyNameplates")

  -- DBM events
  self:RegisterEvent("BossMod_ShowNameplateAura", "DBMShowAura")
  self:RegisterEvent("BossMod_HideNameplateAura", "DBMHideAura")
  self:RegisterEvent("BossMod_EnableFriendlyNameplates", "DBMEnableFriendlyNameplates")
  self:RegisterEvent("BossMod_DisableFriendlyNameplates", "DBMDisableFriendlyNameplates")
end

function OnDisable(self)

end


function GetUnit(self, guid)
  return self.unitsCache[guid]
end

function BuildUnitsCache(self, size)
  for i = 1, size do
    local unit = "raid"..i
    local guid = UnitGUID(unit)
    self.unitsCache[guid] = unit
  end
end

function ClearUnitsCache(self)
   wipe(self.unitsCache)
end


function GetUnitByFrame(self, frame)
    if frame.namePlateUnitToken then
      return frame.namePlateUnitToken
    -- @NOTE : Needed for the ElVUI users
    elseif frame.UnitFrame and frame.UnitFrame.unit then
      return frame.UnitFrame.unit
    end
end

__SystemEvent__()
function NAME_PLATE_CREATED(frame)
  Nameplate(frame)
end

__SystemEvent__()
function NAME_PLATE_UNIT_ADDED(unit)
  local frame = C_NamePlate.GetNamePlateForUnit(unit)
  local guid = UnitGUID(unit)
  local aura = _Addon.auras[guid]

  if aura then
    frame.BMN:UpdateFromAura(aura)
    frame.BMN:Show()
  end
end

__SystemEvent__()
function NAME_PLATE_UNIT_REMOVED(unit)
  local frame = C_NamePlate.GetNamePlateForUnit(unit)
  frame.BMN:Hide()
end


__SystemEvent__()
function ENCOUNTER_START(encounterID, encounterName, difficulty, size)
  Trace(string.format("ENCOUNTER_START : %i, %s, %i, %i", encounterID, encounterName, difficulty, size))
  _Addon.isInEncounter = true
  _Addon.encounterID = encounterID
  _Addon.encounterName = encounterName
  _Addon.encounterDifficulty = difficulty
  _Addon.encounterSize = size

  _Addon:BuildUnitsCache(size)
end

__SystemEvent__()
function ENCOUNTER_END(encounterID, encounterName, difficulty, size, endStatus)
  _Addon.isInEncounter = false
  _Addon:ClearNameplateAuras()

  _Addon:ClearUnitsCache()
end

-- ========================================================================== --
-- Nameplate function
-- ========================================================================== --
function ShowNameplateAura(self, aura)
  if not aura.unit then return end

  self.auras[aura.guid] = aura

  local frame = C_NamePlate.GetNamePlateForUnit(aura.unit)

  if frame then
    frame.BMN:UpdateFromAura(aura)
    frame.BMN:Show()
  end
end

function HideNameplateAura(self, guid)
  local aura = self.auras[guid]
  if aura and aura.unit then
    local frame = C_NamePlate.GetNamePlateForUnit(aura.unit)
    if frame then
      frame.BMN:Hide()
    end
    self.auras[guid] = nil
    aura = nil
  end
end

function ClearNameplateAuras(self)
  for guid, _ in self.auras:GetIterator() do
    self:HideNameplateAura(guid)
  end
end

local previousValue
function EnableFriendlyNameplates()
  NoCombat(function()
    Debug("Enable Friendly Nameplates")
    previousValue = GetCVar("nameplateShowFriends")
    SetCVar("nameplateShowFriends",1)
  end)
end

function DisableFriendlyNameplates()
  NoCombat(function()
    Debug("Disable friendly nameplates")
    SetCVar("nameplateShowFriends", previousValue)
  end)
end

-- ========================================================================== --
-- BigWigs Support
-- ========================================================================== --
function BigWigsShowAura(_, playerName, icon, duration, desaturate)
  Trace("Show Aura called by BigWigs")
  local aura = Aura()
  aura.guid = UnitGUID(playerName)
  aura.unit = playerName
  aura.duration = duration or 0
  aura.desaturated = desaturate
  aura.icon = icon
  aura.startTime = GetTime()
  aura.endTime = aura.startTime + aura.duration
  aura.borderColor = { r = 0, g = 0, b = 0 }

  if _Addon.isInEncounter then
    Encounter.ApplyCustomChange(aura, _Addon.encounterID, _Addon.encounterDifficulty, BossMod.BigWigs)
  end

  -- @TODO Remove this part, by default the border color must be red
  --if desaturate then
    --aura.borderColor = { r = 1, g = 0, b = 0}
  --else
  --aura.borderColor = { r = 0, g = 1, b = 0 }
  --end

  _Addon:ShowNameplateAura(aura)
end

function BigWigsHideAura(_, playerName)
  Trace("Hide Aura called by BigWigs")
  _Addon:HideNameplateAura(UnitGUID(playerName))
end

function BigWigsEnableFriendlyNameplates()
  Debug("[BigWigs] Enable friendly nameplates")
  _Addon:EnableFriendlyNameplates()
end

function BigWigsDisableFriendlyNameplates()
  Debug("[BigWigs] Disable friendly nameplates")
  _Addon:DisableFriendlyNameplates()
end

-- ========================================================================== --
-- DBM Support
-- ========================================================================== --
function DBMShowAura(UnitGUID, currentTexture, duration)
  Trace(string.format("Show Aura called by DBM"))

  local aura = Aura()
  aura.guid = UnitGUID
  aura.unit =  _Addon.isInEncounter and _Addon:GetUnit(UnitGUID)
  aura.duration = duration or 0
  aura.desaturated = false
  aura.icon = currentTexture
  aura.startTime = GetTime()
  aura.endTime = aura.startTime +  aura.duration
  aura.borderColor = { r = 0, g = 0, b = 0 }

  _Addon:ShowNameplateAura(aura)
end

function DBMHideAura(UnitGUID)
  Trace(string.format("Hide Aura called by DBM"))

  _Addon:HideNameplateAura(UnitGUID)
end

function DBMEnableFriendlyNameplates()
  Debug("[DBM] Enable friendly nameplates")
  _Addon:EnableFriendlyNameplates()
end

function DBMDisableFriendlyNameplates()
  Debug("[DBM] Disable friendly nameplates")
  _Addon:DisableFriendlyNameplates()
end

-- ========================================================================== --
--  Convert the AceEvent messages to Scorpio messages
-- =========================================================================  --
function InitiliazeBossMods(self)
  local BigWigsEvents = {
    "BigWigs_EnableFriendlyNameplates", "BigWigs_DisableFriendlyNameplates",
    "BigWigs_ShowNameplateAura", "BigWigs_HideNameplateAura"
    --"BigWigs_ModulePackLoaded"
  }
  local DBMEvents = {
    "BossMod_EnableFriendlyNameplates", "BossMod_DisableFriendlyNameplates",
    "BossMod_ShowNameplateAura", "BossMod_HideNameplateAura"
  }

  if BigWigsLoader then
    Debug("BigWigs Loaded")
    for _, event in ipairs(BigWigsEvents) do
      BigWigsLoader.RegisterMessage(_BossModHandlers, event, "ConvertToScorpioMsg")
    end
  end

  if DBM then
    Debug("DBM Loaded")
    for _, event in ipairs(DBMEvents) do
      DBM:RegisterCallback(event, function(msg, ...) _BossModHandlers:ConvertToScorpioMsg(msg, ...) end)
    end
  end

end

function _BossModHandlers:ConvertToScorpioMsg(e, ...)
  FireSystemEvent(e, ...)
end


-- ========================================================================== --
-- Options
-- ========================================================================== --
__SlashCmd__ "bmn" "config" "- open the options"
function LoadOptions()
  if IsAddOnLoaded("BossModNameplates_Options") then return end

  local loaded, reason = LoadAddOn("BossModNameplates_Options")

  if not loaded then
    Error(string.format("The options cannot be loaded (reason : %s)", reason))
  end
end


local inConfigMode = false
__SlashCmd__ "bmn" "configmode" "- toggle the config mode"
function ToggleConfigMode()
  inConfigMode = not inConfigMode

  if inConfigMode then _Addon:StartConfigMode() else _Addon:StopConfigMode() end

end

function StopConfigMode()
  _Addon:ClearNameplateAuras()
end

function StartConfigMode()
  _Addon:ClearNameplateAuras()
  local spells = List(205429, 205445, 216345, 216344, 227498, 227491, 227500, 227499)

  for _, frame in pairs(C_NamePlate.GetNamePlates()) do
    local unit = _Addon:GetUnitByFrame(frame)
    if UnitIsFriend("player", unit) and UnitIsPlayer(unit) then
      local aura = Aura()
      aura.unit = unit
      aura.guid = UnitGUID(unit)
      aura.icon = GetSpellTexture(spells[math.random(1, spells.Count)])
      aura.duration = 60
      aura.startTime = GetTime()
      aura.endTime = aura.startTime + aura.duration
      aura.desaturated = math.random(1, 2) == 1 and true or false
      aura.borderColor = { r = 0, g = 0, b = 0}

      _Addon:ShowNameplateAura(aura)
    end
  end

end

-- ========================================================================== --
-- Debug
-- ========================================================================== --
__Thread__() __SlashCmd__ "bmn" "debugaura"
function DebugTestAura(arg, ...)
  Debug("Add Aura for test")
  local spell = type(arg) == "string" and tonumber(arg)

  local aura = Aura()
  aura.unit = "target"
  aura.guid = UnitGUID("target")
  aura.duration = 10
  aura.desaturated = true
  --aura.icon = GetSpellTexture(205429)
  aura.icon =  spell and GetSpellTexture(spell) or GetSpellTexture(205429)
  aura.startTime = GetTime()
  aura.endTime = aura.startTime + aura.duration
  aura.borderColor = { r = 0, g = 0, b =  0 }


  Encounter.ApplyCustomChange(aura,
      Encounter.Legion.Nighthold.Etraeus,
      Difficulty.Mythic,
      BossMod.BigWigs)

  --if desaturate then
    --aura.borderColor = { r = 1, g = 0, b = 0}
  --else
    --aura.borderColor = { r = 0, g = 1, b = 0 }
  --end

  _Addon:ShowNameplateAura(aura)

  Delay(aura.duration)
  _Addon:HideNameplateAura(aura.guid)
  Debug("Remove Aura for test")

end
